/*
 * Copyright (c) 2005, 2006 Sharp Corporation
 */

#include <direct/messages.h>

#include "shdfb_gfx.h"
#include "shdfb_accel.h"

bool shdfbFillRectangle (void  *pDriver_data, void *pDevice_data,  DFBRectangle *ptRect)
{
	dShdfbDeviceData_t *ptDevice = (dShdfbDeviceData_t *) pDevice_data;
	int32 iRet;
	
	if (ptDevice->bDisableAlpha) {
		ptDevice->uiColor |= 0xFF;
	};
	
	if (ptDevice->uiColor == 0) {
		return true;
	}

#ifndef EMULATION	
	iRet = iLnxDFBFill (ptRect->x, ptRect->y, ptRect->w, ptRect->h, ptDevice->pvDestAddr, ptDevice->uiColor, &ptDevice->tDestImage);
#else
	iRet = 0;
#endif
    return iRet == 0;
}

bool shdfbBlit (void  *pDriver_data, void *pDevice_data,  DFBRectangle *ptRect, int32 iDx, int32 iDy)
{
	dShdfbDeviceData_t *ptDevice = (dShdfbDeviceData_t *) pDevice_data;
	int32 iRet;

#ifndef EMULATION	
	if (ptDevice->pvSrcAddr == NULL) {
		iRet = iLnxDFBBitBlit (ptRect->x, ptRect->y, ptRect->w, ptRect->h, ptDevice->uiColor,
									iDx, iDy, ptRect->w, ptRect->h, ptDevice->pvDestAddr);
	} else {
		iRet = iLnxDFBBlit (ptRect->x, ptRect->y, ptRect->w, ptRect->h, ptDevice->pvSrcAddr, &ptDevice->tSrcImage,
									iDx, iDy, ptRect->w, ptRect->h, ptDevice->pvDestAddr, &ptDevice->tDestImage, 0, 0);
	}
#else
	iRet = 0;
#endif
    return iRet == 0;
}

bool shdfbStretchBlit (void  *pDriver_data, void *pDevice_data,  DFBRectangle *ptSrect, DFBRectangle *ptDrect)
{
	dShdfbDeviceData_t *ptDevice = (dShdfbDeviceData_t *) pDevice_data;
	int32 iRet;

#ifndef EMULAITON
	int32 fBlendAlpha = 0;
	int32 fDstAlphaFromSrc = 0;

	if (ptDevice->bDisableAlpha == false) {
		fBlendAlpha = 1;
	} else {
		fBlendAlpha = 0;
	}

	if (ptDevice->bDstAlphaFromSrc == true) {
		fDstAlphaFromSrc = 1;
	} else {
		fDstAlphaFromSrc = 0;
	}

	iRet = iLnxDFBBlit(ptSrect->x, ptSrect->y, ptSrect->w, ptSrect->h, ptDevice->pvSrcAddr, &ptDevice->tSrcImage,
					   ptDrect->x, ptDrect->y, ptDrect->w, ptDrect->h, ptDevice->pvDestAddr, &ptDevice->tDestImage,
					   fBlendAlpha, fDstAlphaFromSrc);
	
#else
	iRet = 0;
#endif
    return iRet == 0;
}

void shdfbCheckState (void *pDriver_data, void *pDevice_data, CardState *ptState, DFBAccelerationMask tAccel)
{
#ifndef EMULATION	
    dShdfbDriverData_t *ptDriver = (dShdfbDriverData_t *) pDriver_data;

    if (DFB_DRAWING_FUNCTION (tAccel)) {
    	if (tAccel & DFXL_FILLRECTANGLE) {
	    	ptState->accel |= DFXL_FILLRECTANGLE;
    	} 
    } else {
#define NO_BITBLIT    	
#ifdef NO_BITBLIT
		void *addr = ptState->source->front_buffer->system.addr;
 			
	    if (addr != &ptDriver->tPrimaryAddr && 
	    	addr != &ptDriver->tYUVAddr &&
	    	addr != &ptDriver->tARGBAddr &&
	    	addr != &ptDriver->tMainDisplayAddr) {
	    	return;
	    }
#endif	    
    	if (tAccel & DFXL_BLIT) {
	    	ptState->accel |= DFXL_BLIT;
    	} else if (tAccel & DFXL_STRETCHBLIT) {
	    	ptState->accel |= DFXL_STRETCHBLIT;
        }
    }
#endif    
}

void shdfbSetState (void *pDriver_data, void *pDevice_data, GraphicsDeviceFuncs *ptFuncs, CardState *ptState, DFBAccelerationMask tAccel)
{
#ifndef EMULATION	
	dShdfbDeviceData_t *ptDevice = (dShdfbDeviceData_t *) pDevice_data;
    dShdfbDriverData_t *ptDriver = (dShdfbDriverData_t *) pDriver_data;

 	if (DFB_DRAWING_FUNCTION (tAccel)) {
    	if (tAccel & DFXL_FILLRECTANGLE) {
 			void *addr = ptState->destination->front_buffer->system.addr;
	        ptFuncs->FillRectangle = shdfbFillRectangle;
            ptState->set = DFXL_FILLRECTANGLE;
		    ptDevice->uiColor = (uint32) ptState->color.a | ((uint32) ptState->color.b) << 8 | ((uint32) ptState->color.g) << 16 | ((uint32) ptState->color.r) <<24;
		    if (addr == &ptDriver->tMainDisplayAddr || ptState->blittingflags & DSBLIT_BLEND_ALPHACHANNEL) {
	    		/* ե֥ǥ󥰤ͭˤ */
	    		ptDevice->bDisableAlpha = false;
	    	} else {
	    		ptDevice->bDisableAlpha = true;
	    	}
		    ptDevice->pvDestAddr =addr;
		    ptDevice->tDestImage.uiWidth = ptState->destination->width;
		    ptDevice->tDestImage.uiHeight = ptState->destination->height;
		    ptDevice->tDestImage.uiPitch = ptState->destination->front_buffer->video.pitch;
    	}
 	} else {
 		if (tAccel & DFXL_BLIT) {
 			void *addr = ptState->source->front_buffer->system.addr;
			
	        ptFuncs->Blit = shdfbBlit;
		    ptState->set = DFXL_BLIT;		    
#ifndef NO_BITBLIT 			
		    if (addr != &ptDriver->tPrimaryAddr && addr != &ptDriver->tYUVAddr) {
	 			int iPitch;
	 			
			    ptDevice->pvSrcAddr = NULL;
				dfb_surface_soft_lock( ptState->source, DSLF_WRITE, &addr, &iPitch, 0 );			
			    vSHDFBConvert (addr, ptState->source->width, ptState->source->height, iPitch);
		    	dfb_surface_unlock( ptState->source, 0 );
			    ptDevice->uiColor = (uint32) ptState->color.a | ((uint32) ptState->color.b) << 8 | ((uint32) ptState->color.g) << 16 | ((uint32) ptState->color.r) <<24;
 		    } else {
#endif 		    	
		    	ptDevice->pvSrcAddr = addr;
#ifndef NO_BITBLIT		    	
		    }
#endif		    
	    	ptDevice->pvDestAddr = ptState->destination->front_buffer->system.addr;
	    	if (ptState->blittingflags & DSBLIT_BLEND_ALPHACHANNEL) {
	    		/* ե֥ǥ󥰤ͭˤ */
	    		ptDevice->bDisableAlpha = false;
	    	} else {
	    		ptDevice->bDisableAlpha = true;
	    	}
		    ptDevice->tSrcImage.uiWidth = ptState->source->width;
		    ptDevice->tSrcImage.uiHeight = ptState->source->height;
		    ptDevice->tSrcImage.uiPitch = ptState->source->front_buffer->video.pitch;
		    ptDevice->tDestImage.uiWidth = ptState->destination->width;
		    ptDevice->tDestImage.uiHeight = ptState->destination->height;
		    ptDevice->tDestImage.uiPitch = ptState->destination->front_buffer->video.pitch;
		} else if (tAccel & DFXL_STRETCHBLIT) {
			ptFuncs->StretchBlit = shdfbStretchBlit;
		    ptState->set = DFXL_STRETCHBLIT;
		    ptDevice->pvSrcAddr = ptState->source->front_buffer->system.addr;
		    ptDevice->pvDestAddr = ptState->destination->front_buffer->system.addr;
		    ptDevice->tSrcImage.uiWidth = ptState->source->width;
		    ptDevice->tSrcImage.uiHeight = ptState->source->height;
		    ptDevice->tSrcImage.uiPitch = ptState->source->front_buffer->video.pitch;
		    ptDevice->tDestImage.uiWidth = ptState->destination->width;
		    ptDevice->tDestImage.uiHeight = ptState->destination->height;
		    ptDevice->tDestImage.uiPitch = ptState->destination->front_buffer->video.pitch;

			if (ptState->blittingflags & DSBLIT_BLEND_ALPHACHANNEL) {
				/* ե֥ǥ󥰤ͭˤ */
				ptDevice->bDisableAlpha = false;
			} else {
				ptDevice->bDisableAlpha = true;
			}
			

			if (ptState->blittingflags & DSBLIT_DSTALPHA_FROMSRC) {
				ptDevice->bDstAlphaFromSrc = true;
			} else {
				ptDevice->bDstAlphaFromSrc = false;
			}
 		}
    }

    ptState->modified = 0;
#endif    
}
